function Create(self)
	function APCOffset(Origin, ang, dist)
		if Origin then
			return Vector(Origin.X, Origin.Y) + Vector(dist, 0):RadRotate(ang)
		end
		return Vector(dist, 0):RadRotate(ang)
	end
	
	self.DestructTimer = Timer()
	self.ConcreteTimer = Timer()
	self.GunTimer = Timer()
	
	self.Wheels = {}
	self.WhOfst = {}
	self.Suspension = {}
	self.Barrel = nil
	self.throttle = 0
	self.destruct = 0
	self.maxAmmo = 50	-- MG: Magazine capacity
	self.ammo = self.maxAmmo
	self.rof = 90		-- MG: ms between shots
	self.rot = 2300		-- MG: Reload time in ms
	self.timing = self.rof
	
	-- Find the gun
	local thisID = self.ID
	local MoObj = nil
	
	for i = 1, MovableMan:GetMOIDCount()-1 do
		if MovableMan:GetRootMOID(i) == thisID then	-- an obj without a parent has ID == RootID
			MoObj = MovableMan:GetMOFromID(i)
			if MoObj.ClassName == "HDFirearm" then
				self.Barrel = ToAttachable(MoObj)
			elseif MoObj.ClassName == "Turret" then
				self.Turret = ToAttachable(MoObj)
			end
		end
	end
end

function Destroy(self)
	for i = 1, #self.Wheels do
		if MovableMan:ValidMO(self.Wheels[i]) then
			self.Wheels[i].ToDelete = true
		end
	end
	
	for i = 1, #self.Suspension do
		if MovableMan:ValidMO(self.Suspension[i]) then
			self.Suspension[i].ToDelete = true
		end
	end
end

function Update(self)
	local dist = 32		-- pixels between wheels
	local below = 35	-- pxels between wheels and body
	local multi = 1000	-- force multiplyer
	local maxForce = 3000
	local CenterPos = self.Pos + self:RotateOffset(Vector(0, -5))
	local RightPos = nil
	local LeftPos = nil
	local Ctrl = self:GetController()
	
	if self.HFlipped then
		RightPos = self.Pos + self:RotateOffset(Vector(-dist+10, -5))
		LeftPos = self.Pos + self:RotateOffset(Vector(dist-10, -5))
	else
		RightPos = self.Pos + self:RotateOffset(Vector(dist-10, -5))
		LeftPos = self.Pos + self:RotateOffset(Vector(-dist+10, -5))
	end
	
	if self.Health < 1 then
		self:GibThis()
	end
	
	if Ctrl:IsState(Controller.MOVE_LEFT) and self.throttle < 14 then
		self.throttle = self.throttle + 0.2
	elseif Ctrl:IsState(Controller.MOVE_RIGHT) and self.throttle > -14 then
		self.throttle = self.throttle - 0.2
	else
		self.throttle = self.throttle * 0.98
	end
	
	-- Initialize or verify suspension
	for i = 1, 7 do
		if not MovableMan:ValidMO(self.Suspension[i]) then
			self.Suspension[i] = CreateMOSRotating("Suspension", "EDV.rte")
			self.Suspension[i].Pos = self.Pos
			MovableMan:AddParticle(self.Suspension[i])
		end
	end
		
	-- Update Wheels
	local force = 0
	local Tmp = nil
	for i = 1, 3 do
		if self.HFlipped then
			self.WhOfst[i] = self.Pos + self:RotateOffset(Vector(dist*(i-2), below))
		else
			self.WhOfst[i] = self.Pos + self:RotateOffset(Vector(dist*(2-i), below))
		end
		
		if not MovableMan:ValidMO(self.Wheels[i]) then	-- Initialize or verify wheels
			self.Wheels[i] = CreateMOSRotating("APCWheel", "EDV.rte")
			self.Wheels[i].Pos = self.WhOfst[i]
			self.Wheels[i].Team = self.Team
			MovableMan:AddParticle(self.Wheels[i])
		end
		
		Tmp = self.Wheels[i].Pos - self.WhOfst[i]
		self.Wheels[i].AngularVel = self.throttle
		if Tmp.Magnitude > 20 then
			self.Wheels[i].Pos = self.WhOfst[i]
			self.Wheels[i]:ClearForces()
		elseif Tmp.Magnitude > 4 then
			force = math.min(Tmp.Magnitude * multi, maxForce)
			self.Wheels[i]:AddForce(APCOffset(Vector(0,0), Tmp.AbsRadAngle + 3.141592, Tmp.Magnitude * multi), Vector(0,0))
			
			if self.Vel.Magnitude < 5 and math.abs(self.throttle) > 9.9 and math.abs(self.Wheels[1].AngularVel) > 7 then
				self.Wheels[i]:EraseFromTerrain()	-- Dislodge wheel if necessary
			end
		else
			self.Wheels[i]:MoveOutOfTerrain(6)	-- Sand
		end
	end
	
	-- Update suspension
	self.Suspension[1].Pos = (self.Wheels[2].Pos + CenterPos)/2
	self.Suspension[1].RotAngle = (self.Wheels[2].Pos - CenterPos).AbsRadAngle + 1.57
	
	self.Suspension[2].Pos = (self.Wheels[1].Pos + RightPos)/2
	self.Suspension[2].RotAngle = (self.Wheels[1].Pos - RightPos).AbsRadAngle + 1.57
	
	self.Suspension[3].Pos = (self.Wheels[3].Pos + LeftPos)/2
	self.Suspension[3].RotAngle = (self.Wheels[3].Pos - LeftPos).AbsRadAngle + 1.57
	
	self.Suspension[4].Pos = (self.Wheels[1].Pos + CenterPos)/2
	self.Suspension[4].RotAngle = (self.Wheels[1].Pos - CenterPos).AbsRadAngle + 1.57
	
	self.Suspension[5].Pos = (self.Wheels[3].Pos + CenterPos)/2
	self.Suspension[5].RotAngle = (self.Wheels[3].Pos - CenterPos).AbsRadAngle + 1.57
	
	self.Suspension[6].Pos = (self.Wheels[2].Pos + RightPos)/2
	self.Suspension[6].RotAngle = (self.Wheels[2].Pos - RightPos).AbsRadAngle + 1.57
	
	self.Suspension[7].Pos = (self.Wheels[2].Pos + LeftPos)/2
	self.Suspension[7].RotAngle = (self.Wheels[2].Pos - LeftPos).AbsRadAngle + 1.57
	
	-- Update chassi
	self:MoveOutOfTerrain(6)	-- Sand
	self.AngularVel = self.AngularVel * 0.5
	self.RotAngle = (self.Wheels[1].Pos - self.Wheels[3].Pos).AbsRadAngle
	Tmp = APCOffset((self.Wheels[1].Pos + self.Wheels[2].Pos + self.Wheels[3].Pos)/3, self.RotAngle + 1.57, below) - self.Pos
	
	if self.Vel.Magnitude > 15 then
		local Vel = Vector(self.Vel.X, self.Vel.Y)
		Vel:SetMagnitude(15)
		self.Vel = Vel
	else
		self.Vel = self.Vel * 0.98
	end
	
	if Tmp.Magnitude > 1 then
		force = math.min(Tmp.Magnitude^2 * multi * 1.8, 140000)
		self:AddForce(APCOffset(Vector(0,0), Tmp.AbsRadAngle, force), Vector(0,0))
	end
	
	if self.DestructTimer:IsPastSimMS(600) then
		self.DestructTimer:Reset()
		for i = 1, #self.Wheels do
			if self.Wheels[i].Pos.Y < self.Pos.Y then
				self.destruct = self.destruct + 1
			end
		end
		
		if self.destruct > 9 then	-- The vehicle is on its back, self destruct
			self:GibThis()
		else
			self.destruct = math.max(self.destruct - 1, 0)
		end
	end
	
	local fire = false
	if self:IsPlayerControlled() then
		if self.ammo > 0 and Ctrl:IsState(Controller.BODY_JUMP) and self.GunTimer:IsPastSimMS(self.timing) then
			fire = true
		elseif Ctrl:IsState(Controller.BODY_CROUCH) and self.ConcreteTimer:IsPastSimMS(100) then
			self.ConcreteTimer:Reset()
			
			local Emitter = CreateAEmitter("Concrete Emitter", "EDV.rte")
			Emitter.Vel = self.Vel
			Emitter.Pos = self.Pos + self:RotateOffset(Vector(43, 9))
			
			if self.HFlipped then
				Emitter.RotAngle = self.RotAngle + 3.641592
			else
				Emitter.RotAngle = self.RotAngle - 0.5
			end
			
			MovableMan:AddParticle(Emitter)
		end
	elseif self.ammo > 0 and Ctrl:IsState(Controller.WEAPON_FIRE) and self.GunTimer:IsPastSimMS(self.timing) then
		fire = true	-- let the AI fire the co-ax
	end
	
	if self.Turret:IsAttached() then
		if fire and self.Barrel:IsAttached() then
			self.GunTimer:Reset()
			
			local aim = ToActor(self):GetAimAngle(true)
			if (aim - self.RotAngle) < 4.23 and (aim - self.RotAngle) > -0.69 then
				local bulletType = "MG Flash"
				if self.ammo % 4 == 0 then
					bulletType = "MG Flash Tracer"
				end
				
				local Emitter = CreateAEmitter(bulletType, "EDV.rte")
				Emitter.Vel = self.Vel
				Emitter.RotAngle = aim
				Emitter.Pos = self.Barrel.Pos + self.Barrel:RotateOffset(Vector(1, -10))
				Emitter:SetWhichMOToNotHit(self, -1)
				MovableMan:AddParticle(Emitter)
				
				self.ammo = self.ammo - 1
				if self.ammo < 1 then
					self.timing = self.rot
					self.ammo = self.maxAmmo
					
					Emitter = CreateAEmitter("Magazine Emitter", "EDV.rte")
					Emitter.Vel = self.Vel
					Emitter.Pos = self.Pos + self:RotateOffset(Vector(0, -36))
					
					if self.HFlipped then
						Emitter.RotAngle = self.RotAngle + 1.2
					else
						Emitter.RotAngle = self.RotAngle + 2.1
					end
					
					MovableMan:AddParticle(Emitter)
				else
					self.timing = self.rof
				end
			end
		end
	else
		self:GibThis()	-- no turret
	end
end